@@ -25,66 +25,24 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import pyatspi
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
 
 import orca.messages as messages
 import orca.settings_manager as settings_manager
 import orca.speech_generator as speech_generator
+from orca.ax_component import AXComponent
+from orca.ax_object import AXObject
+from orca.ax_table import AXTable
+from orca.ax_text import AXText
+from orca.ax_utilities import AXUtilities
 
-_settingsManager = settings_manager.getManager()
 
 class SpeechGenerator(speech_generator.SpeechGenerator):
     def __init__(self, script):
         speech_generator.SpeechGenerator.__init__(self, script)
 
-    def __overrideParagraph(self, obj, **args):
-        # Treat a paragraph which is serving as a text entry in a dialog
-        # as a text object.
-        #
-        role = args.get('role', obj.getRole())
-        override = \
-            role == "text frame" \
-            or (role == pyatspi.ROLE_PARAGRAPH \
-                and self._script.utilities.ancestorWithRole(
-                      obj, [pyatspi.ROLE_DIALOG], [pyatspi.ROLE_APPLICATION]))
-        return override
-
-    def _generateRoleName(self, obj, **args):
-        result = []
-        role = args.get('role', obj.getRole())
-        if role == pyatspi.ROLE_TOGGLE_BUTTON \
-           and obj.parent.getRole() == pyatspi.ROLE_TOOL_BAR:
-            pass
-        else:
-            # Treat a paragraph which is serving as a text entry in a dialog
-            # as a text object.
-            #
-            override = self.__overrideParagraph(obj, **args)
-            if override:
-                oldRole = self._overrideRole(pyatspi.ROLE_TEXT, args)
-            # Treat a paragraph which is inside of a spreadsheet cell as
-            # a spreadsheet cell.
-            #
-            elif role == 'ROLE_SPREADSHEET_CELL':
-                oldRole = self._overrideRole(pyatspi.ROLE_TABLE_CELL, args)
-                override = True
-            result.extend(speech_generator.SpeechGenerator._generateRoleName(
-                          self, obj, **args))
-            if override:
-                self._restoreRole(oldRole, args)
-        return result
-
-    def _generateTextRole(self, obj, **args):
-        result = []
-        role = args.get('role', obj.getRole())
-        if role == pyatspi.ROLE_TEXT and obj.parent.getRole() == pyatspi.ROLE_COMBO_BOX:
-            return []
-
-        if role != pyatspi.ROLE_PARAGRAPH \
-           or self.__overrideParagraph(obj, **args):
-            result.extend(self._generateRoleName(obj, **args))
-        return result
-
     def _generateLabel(self, obj, **args):
         """Returns the label for an object as an array of strings (and
         possibly voice and audio specifications).  The label is
@@ -92,10 +50,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         and an empty array will be returned if no label can be found.
         """
         result = []
-        override = self.__overrideParagraph(obj, **args)
         label = self._script.utilities.displayedLabel(obj) or ""
-        if not label and override:
-            label = self._script.utilities.displayedLabel(obj.parent) or ""
+        if not label:
+            label = self._script.utilities.displayedLabel(AXObject.get_parent(obj)) or ""
         if label:
             result.append(label.strip())
             result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
@@ -114,30 +71,30 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         # TODO - JD: This should be the behavior by default. But the default
         # generators call displayedText(). Once that is corrected, this method
         # can be removed.
-        if obj.name:
-            result = [obj.name]
+        if AXObject.get_name(obj):
+            result = [AXObject.get_name(obj)]
             result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
             return result
 
         return super()._generateName(obj, **args)
 
     def _generateLabelAndName(self, obj, **args):
-        if obj.getRole() != pyatspi.ROLE_COMBO_BOX:
+        if AXObject.get_role(obj) != Atspi.Role.COMBO_BOX:
             return super()._generateLabelAndName(obj, **args)
 
         # TODO - JD: This should be the behavior by default because many
         # toolkits use the label for the name.
         result = []
-        label = self._script.utilities.displayedLabel(obj) or obj.name
+        label = self._script.utilities.displayedLabel(obj) or AXObject.get_name(obj)
         if label:
             result.append(label)
             result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
 
-        name = obj.name
+        name = AXObject.get_name(obj)
         if label == name or not name:
             selected = self._script.utilities.selectedChildren(obj)
             if selected:
-                name = selected[0].name
+                name = AXObject.get_name(selected[0])
 
         if name:
             result.append(name)
@@ -145,36 +102,6 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         return result
 
-    def _generateLabelOrName(self, obj, **args):
-        """Gets the label or the name if the label is not preset."""
-
-        result = []
-        override = self.__overrideParagraph(obj, **args)
-        # Treat a paragraph which is serving as a text entry in a dialog
-        # as a text object.
-        #
-        if override:
-            result.extend(self._generateLabel(obj, **args))
-            if len(result) == 0 and obj.parent:
-                parentLabel = self._generateLabel(obj.parent, **args)
-                # If we aren't already focused, we will have spoken the
-                # parent as part of the speech context and do not want
-                # to repeat it.
-                #
-                alreadyFocused = args.get('alreadyFocused', False)
-                if alreadyFocused:
-                    result.extend(parentLabel)
-                # If we still don't have a label, look to the name.
-                #
-                if not parentLabel and obj.name and len(obj.name):
-                    result.append(obj.name)
-                if result:
-                    result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
-        else:
-            result.extend(speech_generator.SpeechGenerator._generateLabelOrName(
-                self, obj, **args))
-        return result
-
     def _generateAnyTextSelection(self, obj, **args):
         comboBoxEntry = self._script.utilities.getEntryForEditableComboBox(obj)
         if comboBoxEntry:
@@ -202,28 +129,29 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if that description is different from that of the name and
         label.
         """
-        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+        if settings_manager.getManager().getSetting('onlySpeakDisplayedText'):
             return []
 
-        if not _settingsManager.getSetting('speakDescription'):
+        if not settings_manager.getManager().getSetting('speakDescription'):
             return []
 
         if not args.get('formatType', '').endswith('WhereAmI'):
             return []
 
         result = []
-        if obj.description:
+        description = AXObject.get_description(obj)
+        if description:
             # The description of some OOo paragraphs consists of the name
             # and the displayed text, with punctuation added. Try to spot
             # this and, if found, ignore the description.
             #
             text = self._script.utilities.displayedText(obj) or ""
-            desc = obj.description.replace(text, "")
-            for item in obj.name.split():
+            desc = description.replace(text, "")
+            for item in AXObject.get_name(obj).split():
                 desc = desc.replace(item, "")
             for char in desc.strip():
                 if char.isalnum():
-                    result.append(obj.description)
+                    result.append(description)
                     break
 
         if result:
@@ -231,12 +159,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         return result
 
     def _generateCurrentLineText(self, obj, **args):
-        if self._script.utilities.isTextDocumentCell(obj.parent):
-            priorObj = args.get('priorObj', None)
-            if priorObj and priorObj.parent != obj.parent:
-                return []
-
-        if obj.getRole() == pyatspi.ROLE_COMBO_BOX:
+        if AXObject.get_role(obj) == Atspi.Role.COMBO_BOX:
             entry = self._script.utilities.getEntryForEditableComboBox(obj)
             if entry:
                 return super()._generateCurrentLineText(entry)
@@ -257,66 +180,19 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         """Treat toggle buttons in the toolbar specially. This is so we can
         have more natural sounding speech such as "bold on", "bold off", etc."""
         result = []
-        role = args.get('role', obj.getRole())
-        if role == pyatspi.ROLE_TOGGLE_BUTTON \
-           and obj.parent.getRole() == pyatspi.ROLE_TOOL_BAR:
-            if obj.getState().contains(pyatspi.STATE_CHECKED):
+        role = args.get('role', AXObject.get_role(obj))
+        if role == Atspi.Role.TOGGLE_BUTTON \
+           and AXObject.get_role(AXObject.get_parent(obj)) == Atspi.Role.TOOL_BAR:
+            if AXUtilities.is_checked(obj):
                 result.append(messages.ON)
             else:
                 result.append(messages.OFF)
             result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
-        elif role == pyatspi.ROLE_TOGGLE_BUTTON:
+        elif role == Atspi.Role.TOGGLE_BUTTON:
             result.extend(speech_generator.SpeechGenerator._generateToggleState(
                 self, obj, **args))
         return result
 
-    def _generateRowHeader(self, obj, **args):
-        """Returns an array of strings (and possibly voice and audio
-        specifications) that represent the row header for an object
-        that is in a table, if it exists.  Otherwise, an empty array
-        is returned. Overridden here so that we can get the dynamic
-        row header(s).
-        """
-
-        if self._script.utilities.shouldReadFullRow(obj):
-            return []
-
-        newOnly = args.get('newOnly', False)
-        rowHeader, columnHeader = \
-            self._script.utilities.getDynamicHeadersForCell(obj, newOnly)
-        if not rowHeader:
-            return super()._generateRowHeader(obj, **args)
-
-        result = []
-        text = self._script.utilities.displayedText(rowHeader)
-        if text:
-            result.append(text)
-            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
-
-        return result
-
-    def _generateColumnHeader(self, obj, **args):
-        """Returns an array of strings (and possibly voice and audio
-        specifications) that represent the column header for an object
-        that is in a table, if it exists.  Otherwise, an empty array
-        is returned. Overridden here so that we can get the dynamic
-        column header(s).
-        """
-
-        newOnly = args.get('newOnly', False)
-        rowHeader, columnHeader = \
-            self._script.utilities.getDynamicHeadersForCell(obj, newOnly)
-        if not columnHeader:
-            return super()._generateColumnHeader(obj, **args)
-
-        result = []
-        text = self._script.utilities.displayedText(columnHeader)
-        if text:
-            result.append(text)
-            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
-
-        return result
-
     def _generateTooLong(self, obj, **args):
         """If there is text in this spread sheet cell, compare the size of
         the text within the table cell with the size of the actual table
@@ -325,44 +201,39 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         Returns an indication of how many characters are greater than the size
         of the spread sheet cell, or None if the message fits.
         """
-        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+        if settings_manager.getManager().getSetting('onlySpeakDisplayedText'):
             return []
 
+        # TODO - JD: Can this be moved to AXText?
         result = []
-        try:
-            text = obj.queryText()
-            objectText = \
-                self._script.utilities.substring(obj, 0, -1)
-            extents = obj.queryComponent().getExtents(pyatspi.DESKTOP_COORDS)
-        except NotImplementedError:
-            pass
-        else:
-            tooLongCount = 0
-            for i in range(0, len(objectText)):
-                [x, y, width, height] = text.getRangeExtents(i, i + 1, 0)
-                if x < extents.x:
-                    tooLongCount += 1
-                elif (x + width) > extents.x + extents.width:
-                    tooLongCount += len(objectText) - i
-                    break
-            if tooLongCount > 0:
-                result = [messages.charactersTooLong(tooLongCount)]
+        length = AXText.get_character_count(obj)
+        tooLongCount = 0
+        extents = AXComponent.get_rect(obj)
+        for i in range(length):
+            rect = AXText.get_character_rect(obj, i)
+            if rect.x < extents.x:
+                tooLongCount += 1
+            elif rect.x + rect.width > extents.x + extents.width:
+                tooLongCount += length - i
+                break
+        if tooLongCount > 0:
+            result = [messages.charactersTooLong(tooLongCount)]
         if result:
             result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
         return result
 
     def _generateHasFormula(self, obj, **args):
-        inputLine = self._script.utilities.locateInputLine(obj)
-        if not inputLine:
+        formula = AXTable.get_cell_formula(obj)
+        if not formula:
             return []
 
-        text = self._script.utilities.displayedText(inputLine)
-        if text and text.startswith("="):
+        if args.get("formatType") == "basicWhereAmI":
+            result = [f"{messages.HAS_FORMULA}. {formula}"]
+        else:
             result = [messages.HAS_FORMULA]
-            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
-            return result
 
-        return []
+        result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
+        return result
 
     def _generateRealTableCell(self, obj, **args):
         """Get the speech for a table cell. If this isn't inside a
@@ -381,22 +252,22 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         result = super()._generateRealTableCell(obj, **args)
 
         if not self._script.utilities.isSpreadSheetCell(obj):
-            if self._script._lastCommandWasStructNav:
+            if self._script.getTableNavigator().last_input_event_was_navigation_command():
                 return result
 
-            if _settingsManager.getSetting('speakCellCoordinates'):
-                result.append(obj.name)
+            if settings_manager.getManager().getSetting('speakCellCoordinates'):
+                result.append(AXObject.get_name(obj))
             return result
 
         isBasicWhereAmI = args.get('formatType') == 'basicWhereAmI'
-        speakCoordinates = _settingsManager.getSetting('speakSpreadsheetCoordinates')
-        if speakCoordinates and not isBasicWhereAmI:
-            result.append(self._script.utilities.spreadSheetCellName(obj))
+        speakCoordinates = settings_manager.getManager().getSetting('speakSpreadsheetCoordinates')
+        if speakCoordinates or isBasicWhereAmI:
+            label = AXTable.get_label_for_cell_coordinates(obj) \
+                or self._script.utilities.spreadSheetCellName(obj)
+            result.append(label)
 
-        if self._script.utilities.shouldReadFullRow(obj):
-            row, col, table = self._script.utilities.getRowColumnAndTable(obj)
-            lastRow = self._script.pointOfReference.get("lastRow")
-            if row != lastRow:
+        if self._script.utilities.shouldReadFullRow(obj, args.get('priorObj')):
+            if self._script.utilities.cellRowChanged(obj):
                 return result
 
         tooLong = self._generateTooLong(obj, **args)
@@ -409,10 +280,14 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             result.extend(self._generatePause(obj, **args))
             result.extend(hasFormula)
 
+        if result == speech_generator.PAUSE:
+            result = [messages.BLANK]
+            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
+
         return result
 
     def _generateTableCellRow(self, obj, **args):
-        if not self._script.utilities.shouldReadFullRow(obj):
+        if not self._script.utilities.shouldReadFullRow(obj, args.get('priorObj')):
             return self._generateRealTableCell(obj, **args)
 
         if not self._script.utilities.isSpreadSheetCell(obj):
@@ -436,22 +311,23 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         dialog).
         """
 
-        if self._script._lastCommandWasStructNav or self._script.inSayAll():
+        if self._script.getTableNavigator().last_input_event_was_navigation_command() \
+           or self._script.inSayAll():
             return []
 
         topLevel = self._script.utilities.topLevelObject(obj)
-        if topLevel and topLevel.getRole() == pyatspi.ROLE_DIALOG:
+        if topLevel and AXObject.get_role(topLevel) == Atspi.Role.DIALOG:
             return []
 
         return super()._generateEndOfTableIndicator(obj, **args)
 
     def _generateNewAncestors(self, obj, **args):
         priorObj = args.get('priorObj', None)
-        if not priorObj or priorObj.getRoleName() == 'text frame':
+        if not priorObj or AXObject.get_role_name(priorObj) == 'text frame':
             return []
 
         if self._script.utilities.isSpreadSheetCell(obj) \
-           and self._script.utilities.isDocumentPanel(priorObj.parent):
+           and self._script.utilities.isDocumentPanel(AXObject.get_parent(priorObj)):
             return []
 
         return super()._generateNewAncestors(obj, **args)
@@ -461,7 +337,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         specifications) that represent the text of the ancestors for
         the object being left."""
 
-        if obj.getRoleName() == 'text frame':
+        if AXObject.get_role_name(obj) == 'text frame':
             return []
 
         priorObj = args.get('priorObj', None)
@@ -471,34 +347,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         return super()._generateOldAncestors(obj, **args)
 
     def _generateUnselectedCell(self, obj, **args):
-        if self._script.utilities.isSpreadSheetCell(obj):
-            return []
-
-        if self._script._lastCommandWasStructNav:
+        if not self._script.utilities.isGUICell(obj):
             return []
 
         return super()._generateUnselectedCell(obj, **args)
-
-    def generateSpeech(self, obj, **args):
-        result = []
-        if args.get('formatType', 'unfocused') == 'basicWhereAmI' \
-           and self._script.utilities.isSpreadSheetCell(obj):
-            oldRole = self._overrideRole('ROLE_SPREADSHEET_CELL', args)
-            # In addition, if focus is in a cell being edited, we cannot
-            # query the accessible table interface for coordinates and the
-            # like because we're temporarily in an entirely different object
-            # which is outside of the table. This makes things difficult.
-            # However, odds are that if we're doing a whereAmI in a cell
-            # which we are editing, we have some pointOfReference info
-            # we can use to guess the coordinates.
-            #
-            args['guessCoordinates'] = obj.getRole() == pyatspi.ROLE_PARAGRAPH
-            result.extend(super().generateSpeech(obj, **args))
-            del args['guessCoordinates']
-            self._restoreRole(oldRole, args)
-        else:
-            oldRole = self._overrideRole(self._getAlternativeRole(obj, **args), args)
-            result.extend(super().generateSpeech(obj, **args))
-            self._restoreRole(oldRole, args)
-
-        return result
